import sys

from django.core.management.base import BaseCommand
from django.core.cache import cache

class Command(BaseCommand):
    help = "delete cache with key name"

    def handle(self, *args, **options):
        if len(args) == 0:
            sys.stdout.write('please input key name.')
            return
        for key in args:
            val = cache.get(key)
            if val is not None:
                cache.delete(key)
                sys.stdout.write('deleting cache: %s, %s' % (key, val))



#:coding=utf-8:
from __future__ import absolute_import

from django import template
from django.template.defaultfilters import stringfilter
from django.utils.encoding import force_text

from .compat import abbrev as abbrev_

register = template.Library()

def abbrev(val, num=20):
    return abbrev_(val, num)
abbrev = stringfilter(abbrev)
register.filter(abbrev)

def cat(value, arg): 
  """ 
  Concatenates value with argument 
  """ 
  return u"%s%s" % (value, force_text(arg))
cat.is_safe=True 
cat = stringfilter(cat)
register.filter(cat)



#:coding=utf-8:

from django.contrib import admin

class DatedModelAdmin(admin.ModelAdmin):
    u"""
    DatedModel のサブクラスの管理画面用のベースクラス

    system_fields という設定を追加する。
    system_fields は管理用のフィールドをタップルで定義します。
    定義するフィールドは「システム管理」というフィールドセットで
    分けて、デフォールトで隠します。

    declared_fieldsets を定義すれば、system_fields は無視されます

    system_fields のデフォールトは::
    
        system_fields = ('ctime', 'utime')

    デフォールト設定を外したい場合は None を設定すれば、
    外せます::

        system_fields = None

    使い方::

        class MyModelAdmin(DatedModelAdmin):
            list_display = ('myfield', 'del_flg', 'utime', 'ctime')
            system_fields = ('my_system_field',) + DatedModelAdmin.system_fields
            #...
    """
    date_hierarchy = 'ctime'
    list_display = ('__str__', 'utime', 'ctime')
    readonly_fields = ('ctime', 'utime')

    system_fields = ('ctime', 'utime')

    def get_fieldsets(self, request, obj=None):
        u"""
        デフォールトで
        """
        if self.declared_fieldsets:
            return self.declared_fieldsets
        if self.system_fields is None:
            return super(DatedModelAdmin, self).get_fieldsets(request, obj)

        form = self.get_form(request, obj)
        fields = list(form.base_fields.keys()) + list(self.get_readonly_fields(request, obj))
        normal_fields = [f for f in fields if f not in self.system_fields]

        return (
            (None, {'fields': normal_fields}),
            (u"システム管理", {
                'classes': ('collapse',),
                'fields': self.system_fields, 
            }),
        )

class BaseModelAdmin(DatedModelAdmin):
    u"""
    BaseModel のサブクラスの管理画面用のベースクラス

    使い方:: 

        class MyModelAdmin(BaseModelAdmin):
            list_display = ('myfield', 'del_flg', 'utime', 'ctime')
    """
    list_display = ('__str__', 'del_flg', 'utime', 'ctime')
    system_fields = ('del_flg', 'ctime', 'utime')



#:coding=utf-8:
import warnings


__all__ = (
    'Views',
)


class Views(object):
    default_name = ''

    def __init__(self, name=None, app_name=None):
        if name is None:
            self.name = self.default_name
        else:
            self.name = name
        if self.name:
            warnings.warn(
                'beproud.django.commons.views.classes.Views: Attribute'
                ' `default_name` and constructor prametor `name` has been'
                ' Removed. It doesn\'t effect anymore.', DeprecationWarning)

        if app_name is None:
            self.app_name = self.default_name
        else:
            self.app_name = app_name

    def get_urls(self):
        return []

    @property
    def urls(self):
        return self.get_urls(), self.app_name



# vim:fileencoding=utf-8

from django.template import Library

from beproud.django.commons.utils.timeutils import relative_time as rel_time

register = Library()

@register.filter
def relative_time(s):
    return rel_time(s) if s else ""



# vim:fileencoding=utf-8

from six import text_type

__all__ = (
    'compare_obj',
    'copy_obj',
)

def compare_obj(base_obj, new_obj, check_primary_key=False, check_related=False, exclude=[], check_related_id=False, display=False):
    """
    モデルインスタンスの比較を行う
    差分を辞書で返す
    """
    diff_dict = {}
    for field in base_obj._meta.fields:
        if field.primary_key and not check_primary_key:
            continue
        if field.rel and not check_related:
            continue
        if field.name in exclude:
            continue
        if field.rel and check_related_id:
            v1 = getattr(base_obj, field.name + '_id')
            v2 = getattr(new_obj, field.name + '_id')
        else:
            v1 = getattr(base_obj, field.name)
            v2 = getattr(new_obj, field.name)
        if v1 != v2:
            if field.choices or field.rel and display:
                if field.rel and v1 is None:
                    _v1 = None
                else:
                    _v1 = getattr(base_obj, field.name)
                if field.rel and v2 is None:
                    _v2 = None
                else:
                    _v2 = getattr(new_obj, field.name)
                diff_dict[field] = (
                    text_type(_v1 or ''),
                    text_type(_v2 or '')
                )
            else:
                diff_dict[field] = (v1, v2)
    if check_related:
        for field in base_obj._meta.many_to_many:
            if field.name in exclude:
                continue
            # base_objが作成されていない場合
            if base_obj.pk:
                m1 = getattr(base_obj, field.name)
                ids1 = list(m1.values_list('id', flat=True))
            else:
                ids1 = []
            m2 = getattr(new_obj, field.name)
            ids2 = list(m2.values_list('id', flat=True))
            if len(ids1) == len(ids2) == len(set(ids1 + ids2)):
                # 変更なし
                continue
            # add attr
            field.many_to_many = True
            # base_objが作成されていない場合
            if base_obj.pk:
                m1_all = m1.all()
            else:
                m1_all = []
            diff_dict[field] = (
                m1_all,
                m2.all()
            )
    return diff_dict

def copy_obj(from_obj, to_obj, check_primary_key=False, check_related=False, exclude=[], copy_related_id=False, check_many_to_many=False):
    """
    モデルインスタンスのフィールドの内容をコピーする
    """
    for field in from_obj._meta.fields:
        if field.primary_key and not check_primary_key:
            continue
        if field.rel and not check_related:
            continue
        if field.name in exclude:
            continue
        if field.rel and copy_related_id:
            v = getattr(from_obj, field.name + '_id')
            setattr(to_obj, field.name + '_id', v)
        else:
            v = getattr(from_obj, field.name)
            setattr(to_obj, field.name, v)
    if check_many_to_many:
        # m2mを設定するため保存する
        to_obj.save()
        for field in from_obj._meta.many_to_many:
            if field.name in exclude:
                # 変更なし
                continue
            m1 = getattr(from_obj, field.name)
            m2 = getattr(to_obj, field.name)
            ids1 = list(m1.values_list('id', flat=True))
            ids2 = list(m2.values_list('id', flat=True))
            if len(ids1) == len(ids2) == len(set(ids1 + ids2)):
                continue
            # コピー m1 -> m2
            m2.clear()
            m2_model = m2.model
            for obj in m1.all():
                m2.add(m2_model.objects.get(pk=obj.pk))



#:coding=utf-8:
from django import forms


def autostrip(cls):
    """
    strip text fields before validation

    example:
    class PersonForm(forms.Form):
        name = forms.CharField(min_length=2, max_length=10)
        email = forms.EmailField()

    PersonForm = autostrip(PersonForm)
    
    #or you can use @autostrip in python >= 2.6

    Author: nail.xx

    Taken from django-annoying
    """
    fields = [(key, value) for key, value in cls.base_fields.items() if isinstance(value, forms.CharField)]
    for field_name, field_object in fields:
        def get_clean_func(original_clean):
            return lambda value: original_clean(value and value.strip())
        clean_func = get_clean_func(getattr(field_object, 'clean'))
        setattr(field_object, 'clean', clean_func)
    return cls
